@@ -1,4 +1,5 @@
 import asyncio
+import logging
 from collections import deque
 from typing import *
 from functools import partial
@@ -6,7 +7,8 @@ from contextlib import asynccontextmanager
 import time
 import socket
 
-import aioredis.connection
+import redis.asyncio as aioredis
+from redis.asyncio import cluster
 
 from celery.backends.redis import (
     RedisBackend as SyncRedisBackend,
@@ -15,9 +17,7 @@ from celery.backends.redis import (
     task_join_will_block,
     states,
     logger,
-    ImproperlyConfigured,
     E_RETRY_LIMIT_EXCEEDED,
-    E_REDIS_SENTINEL_MISSING,
 )
 from celery.backends.asynchronous import (
     Empty,
@@ -36,9 +36,9 @@ from celery.backends.base import (
 from celery.exceptions import TimeoutError
 from celery.result import GroupResult
 
-from celery.aio.redis import get_redis_error_classes
-from celery.aio.utils import retry_over_time
 from celery.aio.result import result_from_tuple
+from celery.patches.kombu.redis import get_redis_error_classes
+from celery.patches.kombu.utils import retry_over_time
 
 
 @register_drainer('aio-default')
@@ -49,7 +49,7 @@ class Drainer(SyncDrainer):
     def __init__(self, result_consumer):
         self.result_consumer = result_consumer
         self._started = False
-        self._drainer: asyncio.Task = None
+        self._drainer: Optional[asyncio.Task] = None
 
     async def drain_events_until(self, r, timeout=None, interval=1, on_interval=None, wait=None):
         wait = wait or self.result_consumer.drain_events
@@ -78,12 +78,12 @@ class Drainer(SyncDrainer):
 
 class ResultConsumer(SyncResultConsumer):
     if TYPE_CHECKING:
-        from aioredis.client import PubSub
+        from redis.asyncio.client import PubSub
 
         backend: 'RedisBackend'
-        _pubsub: PubSub
+        _pubsub: Optional[PubSub]
         drainer: Drainer
-        _running_drainer: asyncio.Task
+        _running_drainer: Optional[asyncio.Task]
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
@@ -112,8 +112,7 @@ class ResultConsumer(SyncResultConsumer):
             ignore_subscribe_messages=True,
         )
         if self.subscribed_to:
-            async with self._pubsub_lock:
-                await self._pubsub.subscribe(*self.subscribed_to)
+            await self._pubsub.subscribe(*self.subscribed_to)
 
     @asynccontextmanager
     async def reconnect_on_error(self):
@@ -175,7 +174,8 @@ class ResultConsumer(SyncResultConsumer):
         async with self.reconnect_on_error():
             async for message in self._pubsub.listen():
                 if message and message['type'] == 'message':
-                    logger.info(f'got message: {message}')
+                    if logger.isEnabledFor(logging.DEBUG):
+                        logger.debug(f'got message: {message}')
                     await self.on_state_change(
                         self._decode_result(message['data']), message
                     )
@@ -251,7 +251,7 @@ class ResultConsumer(SyncResultConsumer):
 class RedisBackend(SyncRedisBackend):
     if TYPE_CHECKING:
         @property
-        def client(self) -> aioredis.Redis: ...
+        def client(self) -> aioredis.Redis: ...  # noqa
         result_consumer: ResultConsumer
 
     ResultConsumer = ResultConsumer
@@ -632,21 +632,12 @@ class RedisBackend(SyncRedisBackend):
 
 
 class SentinelBackend(RedisBackend):
-    try:
-        from aioredis import sentinel
-    except ImportError:
-        sentinel = None
+    sentinel = getattr(aioredis, "sentinel", None)
 
     if TYPE_CHECKING:
-        from aioredis.sentinel import Sentinel
+        from redis.asyncio.sentinel import Sentinel
         def _get_sentinel_instance(self, **params) -> Sentinel: ...
 
-    def __init__(self, *args, **kwargs):
-        if self.sentinel is None:
-            raise ImproperlyConfigured(E_REDIS_SENTINEL_MISSING.strip())
-
-        super().__init__(*args, **kwargs)
-
     def _params_from_url(self, url, defaults):
         chunks = url.split(";")
         connparams = dict(defaults, hosts=[])
@@ -665,3 +656,24 @@ class SentinelBackend(RedisBackend):
 
     _get_sentinel_instance = SyncSentinelBackend._get_sentinel_instance # noqa
     _get_pool = SyncSentinelBackend._get_pool # noqa
+
+
+class ClusterBackend(RedisBackend):
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self.connparams.pop('db')
+
+    def _get_pool(self, **params):
+        return NotImplementedError
+
+    def _create_client(self, **params):
+        return cluster.RedisCluster(**params)
+
+    async def _set(self, key, value):
+        async with self.client.pipeline() as pipe:
+            if self.expires:
+                pipe.setex(key, self.expires, value)
+            else:
+                pipe.set(key, value)
+            await pipe.execute()
+        await self.client.publish(key, value)
