@@ -17,10 +17,10 @@ from celery.app.control import (
     register_after_fork,
     _after_fork_cleanup_control
 )
-from celery.aio.messaging import Producer, Consumer
-from celery.aio.common import maybe_declare
-from celery.aio.exchange import Exchange
-from celery.aio.entity import Queue
+from celery.patches.kombu.messaging import Producer, Consumer
+from celery.patches.kombu.exchange import Exchange
+from celery.patches.kombu.entity import Queue
+from celery.patches.kombu.common import maybe_declare
 
 
 class Mailbox(SyncMailBox):
@@ -163,6 +163,7 @@ class Mailbox(SyncMailBox):
         finally:
             await chan.after_reply_message_received(queue.name)
 
+
 class Inspect(SyncInspect):
     async def _request(self, command, **kwargs):
         reply = await self.app.aio_control.broadcast(
