@@ -30,6 +30,7 @@ except ImportError:  # pragma: no cover
 
 try:
     import redis.sentinel
+    from redis.cluster import RedisCluster
 except ImportError:
     pass
 
@@ -592,3 +593,25 @@ class SentinelBackend(RedisBackend):
             service_name=master_name,
             redis_class=self._get_client(),
         ).connection_pool
+
+
+class ClusterBackend(RedisBackend):
+    """Redis task result store."""
+    redis = RedisCluster
+
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self.connparams.pop('db', None)
+
+    def _set(self, key, value):
+        with self.client.pipeline() as pipe:
+            if self.expires:
+                pipe.setex(key, self.expires, value)
+            else:
+                pipe.set(key, value)
+            pipe.execute()
+        self.client.publish(key, value)
+
+    @cached_property
+    def client(self) -> RedisCluster:
+        return RedisCluster(**self.connparams)
