@@ -17,6 +17,7 @@ import gc
 import os
 import select
 import time
+import sys
 from collections import Counter, deque, namedtuple
 from io import BytesIO
 from numbers import Integral
@@ -27,7 +28,12 @@ from weakref import WeakValueDictionary, ref
 
 from billiard import pool as _pool
 from billiard.compat import buf_t, isblocking, setblocking
-from billiard.pool import ACK, NACK, RUN, TERMINATE, WorkersJoined
+from billiard.pool import (
+    ACK, NACK, RUN, TERMINATE, READY,
+    TASK, EX_OK, EX_FAILURE, EX_RECYCLE, MAXMEM_USED_FMT,
+    WorkersJoined, ExceptionInfo, MaybeEncodingError,
+    mem_rss,
+)
 from billiard.queues import _SimpleQueue
 from kombu.asynchronous import ERR, WRITE
 from kombu.serialization import pickle as _pickle
@@ -63,7 +69,8 @@ except ImportError:  # pragma: no cover
 __all__ = ('AsynPool',)
 
 logger = get_logger(__name__)
-error, debug = logger.error, logger.debug
+error, warning, info, debug = \
+    logger.error, logger.warning, logger.info, logger.debug
 
 UNAVAIL = frozenset({errno.EAGAIN, errno.EINTR})
 
@@ -240,6 +247,86 @@ class Worker(_pool.Worker):
         # is writable.
         self.outq.put((WORKER_UP, (pid,)))
 
+    def workloop(self, debug=debug, now=time.monotonic, pid=None):
+        pid = pid or os.getpid()
+        put = self.outq.put
+        inqW_fd = self.inqW_fd
+        synqW_fd = self.synqW_fd
+        maxtasks = self.maxtasks
+        max_memory_per_child = self.max_memory_per_child or 0
+        prepare_result = self.prepare_result
+
+        wait_for_job = self.wait_for_job
+        _wait_for_syn = self.wait_for_syn
+
+        info = logger.info
+
+        def wait_for_syn(jid):
+            i = 0
+            while 1:
+                if i > 60:
+                    error('!!!WAIT FOR ACK TIMEOUT: job:%r fd:%r!!!',
+                          jid, self.synq._reader.fileno(), exc_info=1)
+                req = _wait_for_syn()
+                if req:
+                    type_, args = req
+                    if type_ == NACK:
+                        return False
+                    assert type_ == ACK
+                    return True
+                i += 1
+
+        completed = 0
+        try:
+            while maxtasks is None or (maxtasks and completed < maxtasks):
+                req = wait_for_job()
+                if req:
+                    type_, args_ = req
+                    assert type_ == TASK
+                    job, i, fun, args, kwargs = args_
+                    info(f"Receive req[{args[1]}]")
+                    put((ACK, (job, i, now(), pid, synqW_fd)))
+                    if _wait_for_syn:
+                        confirm = wait_for_syn(job)
+                        if not confirm:
+                            continue  # received NACK
+                    try:
+                        result = (True, prepare_result(fun(*args, **kwargs)))
+                    except Exception:
+                        result = (False, ExceptionInfo())
+                    try:
+                        put((READY, (job, i, result, inqW_fd)))
+                    except Exception as exc:
+                        _, _, tb = sys.exc_info()
+                        try:
+                            wrapped = MaybeEncodingError(exc, result[1])
+                            einfo = ExceptionInfo((
+                                MaybeEncodingError, wrapped, tb,
+                            ))
+                            put((READY, (job, i, (False, einfo), inqW_fd)))
+                        finally:
+                            del tb
+                    completed += 1
+                    if max_memory_per_child > 0:
+                        used_kb = mem_rss()
+                        if used_kb <= 0:
+                            error('worker unable to determine memory usage')
+                        if used_kb > 0 and used_kb > max_memory_per_child:
+                            warning(MAXMEM_USED_FMT.format(
+                                used_kb, max_memory_per_child))
+                            return EX_RECYCLE
+
+            debug('worker exiting after %d tasks', completed)
+            if maxtasks:
+                return EX_RECYCLE if completed == maxtasks else EX_FAILURE
+            return EX_OK
+        finally:
+            # Before exiting the worker, we want to ensure that that all
+            # messages produced by the worker have been consumed by the main
+            # process. This prevents the worker being terminated prematurely
+            # and messages being lost.
+            self._ensure_messages_consumed(completed=completed)
+
 
 class ResultHandler(_pool.ResultHandler):
     """Handles messages from the pool processes."""
@@ -835,6 +922,8 @@ class AsynPool(_pool.Pool):
                             # keep track of what process the write operation
                             # was scheduled for.
                             proc = job._scheduled_for = fileno_to_inq[ready_fd]
+                            info(f"Schedule task[{job.correlation_id}] "
+                                 f"to <{proc.name}>")
                         except KeyError:
                             # write was scheduled for this fd but the process
                             # has since exited and the message must be sent to
