@@ -8,7 +8,7 @@ from weakref import WeakMethod
 
 from kombu.utils.functional import retry_over_time
 
-from celery.aio.utils import (
+from celery.patches.kombu.utils import (
     retry_over_time as aretry_over_time,
     is_async_callable
 )
